/****************************************************************
**
** Implementation of CIGLWIdget class
**
****************************************************************/


#include "CIGLWidget.h"
#include <QMessageBox.h>

CIGLWidget::CIGLWidget( QWidget *parent, const char *name ): QGLWidget(parent, name)
{
		rotx=roty=0;
		drx=dry=0;
}
	
void CIGLWidget::initializeGL()
{   
  // Set up the rendering context, define display lists etc.:
  glClearColor( 0.0, 1.0, 1.0, 0.0 );
	float spec[4]={.4,.4,.4,1};
	float shin = 1;
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR , spec);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS , shin);
	
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_COLOR_MATERIAL);
	
	// Abilita il la possibilita' di cambiare il alcune componenti colore del 
	// materiale tramite un semplice glcolor, anzich fare glMaterial(...)
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	// Abilita il calcolo dell'illuminazione
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// anche le normali attraversano la pipeline, quindi devono essere  
	// rinormalizzate, prima del calcolo dell'illuminazione, altrimenti 
	// il calcolo dell'illuminazione viene sbagliato.
	glEnable(GL_NORMALIZE); 
	
	// Abilita il test sullo zbuffer e quindi l'eliminazione delle 
	// superfici nascoste
	glEnable(GL_DEPTH_TEST); 	

	// Disabilita lo shading di gouraud per le facce dei poligoni disegnati
	glShadeModel(GL_SMOOTH);
}

void CIGLWidget::resizeGL( int w, int h )
{
  // setup viewport, projection etc.:
	glMatrixMode (GL_PROJECTION);   
	glLoadIdentity ();
	float ratio=(float)w/(float)h;
  gluPerspective(45,ratio,3,20);
	glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
	glMatrixMode (GL_MODELVIEW);
}

void CIGLWidget::paintGL()
{
	glMatrixMode (GL_MODELVIEW);
	glClear(GL_COLOR_BUFFER_BIT |GL_DEPTH_BUFFER_BIT  );
	glLoadIdentity();
	gluLookAt(2,5,12,0,0,0,0,1,0);
	glColor3f(1,.4,.4);
	QString tmp;
//	renderText(lx,ly,tmp.sprintf("Mouse pressed on %i %i",lx,ly));
	glRotatef(roty+dry,0,1,0);
	glRotatef(rotx+drx,1,0,0);
	ring->m.Draw();
}

void CIGLWidget::mousePressEvent(QMouseEvent*e)
{
	sx=e->x();
	sy=e->y();
	update();
}

void CIGLWidget::mouseMoveEvent(QMouseEvent*e)
{
	drx=(float(e->y()-sy)/size().height())*180.0f;
	dry=(float(e->x()-sx)/size().width())*180.0f;
	update();
}

void CIGLWidget::mouseReleaseEvent(QMouseEvent*e)
{
 // QString buf;
	//QMessageBox::information(this,"qt",buf.sprintf("released %f %f",drx,dry));
	rotx+=drx;
	roty+=dry;
	drx=0;
	dry=0;
	update();
}
